<?php

namespace Drupal\castorcito\Plugin\Field\FieldFormatter;

use Drupal\castorcito\CastorcitoManager;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Serialization\Json;

/**
 * Plugin implementation of the 'castorcito_component_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "castorcito_component_formatter",
 *   label = @Translation("Castorcito component"),
 *   field_types = {
 *     "json",
 *     "json_native",
 *     "json_native_binary",
 *   }
 * )
 */
class CastorcitoComponentFormatter extends FormatterBase {

  protected const CONTAINERS = ['container', 'advanced_container'];

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * The castorcito manager.
   *
   * @var Drupal\castorcito\CastorcitoManager
   */
  protected $castorcitoManager;

  /**
   * Constructs a new CastorcitoComponentFormatter.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param Drupal\castorcito\CastorcitoManager $castorcito_manager
   *   The castorcito manager.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    EntityDisplayRepositoryInterface $entity_display_repository,
    CastorcitoManager $castorcito_manager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->entityDisplayRepository = $entity_display_repository;
    $this->castorcitoManager = $castorcito_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_display.repository'),
      $container->get('castorcito.manager')
    );
  }

  /**
   * {@inheritdoc}
  */
  public static function defaultSettings() {
    return [
      'display_settings_components' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
  */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $display_settings_components = $this->getSetting('display_settings_components');
    $widget_config = $this->entityDisplayRepository->getFormDisplay(
      $this->fieldDefinition->get('entity_type'),
      $this->fieldDefinition->get('bundle')
    )->getComponent($this->fieldDefinition->getName());

    if (!empty($widget_config) && $widget_config['type'] === 'castorcito_component_widget') {
      $component_ids = array_filter($widget_config['settings']['components']);

      if ($components  = $this->validateHasDisplaySettings($component_ids)) {
        $form['display_settings_components'] = [
          '#type' => 'fieldset',
          '#title' => $this->t('Components'),
        ];

        $this->cfieldDisplayElements($form, $components);
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $display_settings_components = $this->getSetting('display_settings_components');
    $summary = [];

    if (!empty($display_settings_components)) {
      $components_options = $this->castorcitoManager->castorcitoComponentsOptions();
      $items = [];
      foreach ($display_settings_components as $con_key => $config) {
        $items[] = $components_options[$con_key];
      }

      $summary[] = $this->t('<strong>Display settings</strong>');
      $summary[] = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#items' => $items,
      ];
    }
    else {
      $summary[] = $this->t('Default settings.');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $display_settings_components = $this->getSetting('display_settings_components');
    $elements = [];

    if (!empty($items)) {
      foreach ($items as $key => $item) {
        $model = Json::decode($item->value);
        $display_settings = [];
        $component = $this->validateHasDisplaySettings([$model['type']]);
        if (!empty($component)) {
          $display_settings = $this->getDisplaySettings($component);
        }

        $elements[] = [
          '#theme' => 'castorcito_component_formatter',
          '#component_data' => $model,
          '#display_settings' => $display_settings,
        ];
      }
    }

    return $elements;
  }

  /**
   * Function to build element field display.
   */
  public function cfieldDisplayElements(array &$form, array $components, $parent_keys = []) {
    $formatter_settings = $this->getSetting('display_settings_components');
    foreach ($components as $c_key => $component) {
      $field_configurations = $component->get('field_configuration');
      $settings = $component->get('display_settings');
      $current_keys = [...$parent_keys, $c_key];

      $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $current_keys, [
        '#type' => 'details',
        '#title' => $component->label(),
      ]);

      if (!empty($settings)) {
        $option_current_keys = [...$current_keys, 'display_options'];
        $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $option_current_keys, [
          '#type' => 'details',
          '#title' => $this->t('Display options'),
          '#open' => TRUE,
        ]);

        foreach ($settings as $s_key => $setting) {
          $option_value_keys = [...$option_current_keys, $s_key];
          $default_value = $this->castorcitoManager->getNestedArrayValue($formatter_settings, $option_value_keys, $setting['default_value']);
          $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $option_current_keys, [$s_key => $this->fieldDisplayOption($setting, $option_current_keys, $default_value)]);
        }
      }

      $cfields_current_keys = [...$current_keys, 'cfields'];
      $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $cfields_current_keys, [
        '#type' => 'fieldset',
        '#title' => $this->t('CFields'),
      ]);

      foreach ($field_configurations as $f_key => $configuration) {
        $cfield_current_keys = [...$cfields_current_keys, $f_key];

        if (in_array($configuration['id'], self::CONTAINERS)) {
          $container_field_current_keys = $cfield_current_keys;
          $children_components = $this->validateHasDisplaySettings($configuration['settings']['allowed_children']);

          if (!empty($children_components)) {
            $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $cfield_current_keys, [
              '#type' => 'details',
              '#title' => $this->t($configuration['label']),
            ]);
          }

          if ($configuration['id'] === 'advanced_container') {
            if (!empty($configuration['settings']['head_component'])) {
              $field_current_keys_head = [...$cfield_current_keys, 'head'];
              $head_children_components = $this->validateHasDisplaySettings([$configuration['settings']['head_component']]);

              if (!empty($head_children_components)) {
                $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $field_current_keys_head, [
                  '#type' => 'details',
                  '#title' => $this->t('Head'),
                ]);

                $this->cfieldDisplayElements($form, $head_children_components, $field_current_keys_head);
              }
            }

            if (!empty($configuration['settings']['allowed_children'])) {
              $container_field_current_keys = [...$cfield_current_keys, 'body'];
              $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $container_field_current_keys, [
                '#type' => 'details',
                '#title' => $this->t('Body'),
              ]);
            }
          }

          if (!empty($children_components)) {
            $this->cfieldDisplayElements($form, $children_components, $container_field_current_keys);
          }
        }
        else {
          if (!isset($configuration['settings']['display'])) {
            continue;
          }

          $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $cfield_current_keys, [
            '#type' => 'details',
            '#title' => $this->t($configuration['label']),
          ]);

          $default_value = $this->castorcitoManager->getNestedArrayValue($formatter_settings, $cfield_current_keys, $configuration['settings']['display']);
          $elements = $this->buildFieldElements($default_value, $configuration);
          $this->castorcitoManager->setNestedArrayValue($form['display_settings_components'], $cfield_current_keys, $elements);
        }
      }
    }
  }

  /**
   * Retrieves the display settings of the components.
   */
  private function getDisplaySettings($components, $parent_keys = []) {
    $formatter_settings = $this->getSetting('display_settings_components');
    $display_settings = [];
    foreach ($components as $c_key => $component) {
      $current_keys = [...$parent_keys, $c_key];

      if ($settings = $component->get('display_settings')) {
        $option_current_keys = [...$current_keys, 'display_options'];
        $values = [];
        foreach ($settings as $s_key => $setting) {
          $option_value_keys = [...$option_current_keys, $s_key];
          $values[$s_key] = $this->castorcitoManager->getNestedArrayValue($formatter_settings, $option_value_keys, $setting['default_value']);
        }

        $this->castorcitoManager->setNestedArrayValue($display_settings, $option_current_keys, $values);
      }

      if ($cfield_configurations = $component->get('field_configuration')) {
        foreach ($cfield_configurations as $f_key => $configuration) {
          $cfield_current_keys = [...$current_keys, 'cfields', $f_key];

          if (in_array($configuration['id'], self::CONTAINERS)) {
            $container_field_current_keys = $cfield_current_keys;
            $children_components = $this->validateHasDisplaySettings($configuration['settings']['allowed_children']);

            if ($configuration['id'] === 'advanced_container') {
              if (!empty($configuration['settings']['head_component'])) {
                $field_current_keys_head = [...$cfield_current_keys, 'head'];
                $head_children_components = $this->validateHasDisplaySettings([$configuration['settings']['head_component']]);

                if (!empty($head_children_components)) {
                  $display_settings = array_merge_recursive($display_settings, $this->getDisplaySettings($head_children_components, $field_current_keys_head));
                }
              }

              if (!empty($configuration['settings']['allowed_children'])) {
                $container_field_current_keys = [...$cfield_current_keys, 'body'];
              }
            }

            if (!empty($children_components)) {
              $display_settings = array_merge_recursive($display_settings, $this->getDisplaySettings($children_components, $container_field_current_keys));
            }
          }
          else {
            if (!isset($configuration['settings']['display'])) {
              continue;
            }

            $values = $this->castorcitoManager->getNestedArrayValue($formatter_settings, $cfield_current_keys, $configuration['settings']['display']);
            $this->castorcitoManager->setNestedArrayValue($display_settings, $cfield_current_keys, $values);
          }
        }
      }
    }

    return $display_settings;
  }


  /**
   * Generates a form element based on the provided display settings
   *
   * @param array $setting        The option setting.
   * @param array $keys           An array of keys defining the option setting.
   * @param mixed $default_value  The default value.
   *
   * @return array The option element.
   */
  private function fieldDisplayOption(array $setting, array $keys, $default_value) {
    $element = [
      '#type' => $setting['type'],
      '#title' => $this->t($setting['label']),
      '#default_value' => $default_value ?? '',
      '#description' => $setting['description'] ?? '',
      '#required' => $setting['required'] ?? '',
    ];

    if ($setting['type'] === 'select' && !empty($setting['options'])) {
      $element['#options'] = array_column($setting['options'], 'label', 'key');
    }

    if (isset($setting['visibility_condition']) && !empty($setting['visibility_condition'])) {
      $keys_string = '[' . implode('][', $keys) . ']';
      $trigger = [$setting['visibility_condition']['trigger'] => $setting['visibility_condition']['value']];
      if ($setting['visibility_condition']['trigger'] === 'checked' ||  $setting['visibility_condition']['trigger'] === 'unchecked') {
        $trigger = [$setting['visibility_condition']['trigger'] => TRUE];
      }

      $element['#states'] = [
        $setting['visibility_condition']['state'] => [
          ':input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][display_settings_components]' . $keys_string . '[' . $setting['visibility_condition']['conditional_option'] . ']"]' => $trigger,
        ],
      ];
    }

    return $element;
  }

  /**
   * Builds and returns form field elements based on configuration.
   *
   * @param mixed $default_value  The default value for the form fields.
   * @param array $configuration  Configuration options to customize the fields.
   *
   * @return array The generated form elements.
   */
  private function buildFieldElements($default_value, $configuration) {
    switch ($configuration['id']) {
      case 'image':
        $image_style_options = image_style_options(FALSE);
        return [
          'image_style' => [
            '#type' => 'select',
            '#title' => $this->t('Image style'),
            '#options' => $image_style_options,
            '#default_value' => $default_value['image_style'] ?? '',
            '#empty_option' => $this->t('None (original image)'),
          ],
        ];
      case 'entity_reference':
        $view_mode_options = $this->entityDisplayRepository->getViewModeOptions($configuration['settings']['entity_type']);
        return [
          'view_mode' => [
            '#type' => 'select',
            '#title' => $this->t('View mode'),
            '#options' => $view_mode_options,
            '#default_value' => $default_value['view_mode'] ?? '',
            '#empty_option' => $this->t('- Select -'),
          ],
        ];
      case 'boolean':
        return [
          'on_label' => [
            '#type' => 'textfield',
            '#title' => $this->t('"On" label'),
            '#default_value' => $default_value['on_label'] ?? '',
          ],
          'off_label' => [
            '#type' => 'textfield',
            '#title' => $this->t('"Off" label'),
            '#default_value' => $default_value['off_label'] ?? '',
          ],
        ];
      default:
        return [];
    }
  }

  /**
   * Determines whether any components have display settings.
   *
   * @param array $ids
   *   An array of component identifiers.
   *
   * @return array
   *   An array of component that have display settings.
   */
  private function validateHasDisplaySettings(array $ids): array {
    $components = $this->castorcitoManager->castorcitoComponents($ids);
    $allowed_components = [];

    if (!empty($components)) {
      foreach ($components as $components_id => $component) {
        $cfield_configurations = $component->get('field_configuration');
        $settings = $component->get('display_settings');

        if (!empty($settings)) {
          $allowed_components[$components_id] = $component;
          continue;
        }

        if (!empty($cfield_configurations)) {
          foreach ($cfield_configurations as $configuration) {
            if (!empty($configuration['settings']['display'])) {
              $allowed_components[$components_id] = $component;
              continue;
            }
          }
        }
      }
    }

    return $allowed_components;
  }

}
