<?php

namespace Drupal\castorcito\Plugin\Field\FieldWidget;

use Drupal\castorcito\CastorcitoComponentFieldManager;
use Drupal\castorcito\CastorcitoManager;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldFilteredMarkup;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\json_field\Plugin\Field\FieldWidget\JsonTextareaWidget;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'castorcito_component_widget' widget.
 *
 * @FieldWidget(
 *   id = "castorcito_component_widget",
 *   module = "castorcito",
 *   label = @Translation("Castorcito component"),
 *   description = @Translation("Define all components."),
 *   field_types = {
 *     "json",
 *     "json_native",
 *     "json_native_binary"
 *   },
 * )
 */
class CastorcitoComponentWidget extends JsonTextareaWidget {

  protected const CONTAINERS = ['container', 'advanced_container'];

  /**
   * The component field manager.
   *
   * @var \Drupal\castorcito\CastorcitoComponentFieldManager
   */
  protected $componentFieldManager;

  /**
   * The castorcito manager.
   *
   * @var Drupal\castorcito\CastorcitoManager
   */
  protected $castorcitoManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    CastorcitoComponentFieldManager $component_field_manager,
    CastorcitoManager $castorcito_manager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->componentFieldManager = $component_field_manager;
    $this->castorcitoManager = $castorcito_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('plugin.manager.castorcito_component_field'),
      $container->get('castorcito.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'components' => [],
      'components_cfields_settings' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    if (!isset($form['#is_castorcito'])) {
      $form['#is_castorcito'] = 'field--widget-castorcito-component-widget';
    }

    $element['value'] = $element + [
      '#type' => 'textarea',
      '#default_value' => $items[$delta]->value,
      '#attributes' => [
        'target-weight' => $delta,
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $components = $this->castorcitoManager->castorcitoComponentsOptions();
    $element['components'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Components list'),
      '#options' => $components,
      '#default_value' => $this->getSetting('components'),
      '#required' => TRUE,
      '#multiple' => TRUE,
    ];

    $element['components_cfields_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Components configuration'),
    ];

    $this->cfieldSettingsElements($element, $form_state, array_keys($components));

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $components = array_filter($this->getSetting('components'));

    if (!empty($components)) {
      $components_options = $this->castorcitoManager->castorcitoComponentsOptions();
      $items = [];
      foreach ($components as $component) {
        $items[] = $components_options[$component];
      }

      $summary[] = $this->t('<strong>Allowed components</strong>');
      $summary[] = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#items' => $items,
      ];
    }
    else {
      $summary[] = $this->t('You don\'t have any selected components for this field.');
      $summary[] = $this->t('Choose the components you want to use.');
    }

    return $summary;
  }

  /**
   * Builds the configuration form elements for cfields.
   *
   * @param array &$element
   *   A reference to the parent form element array where new elements will be added.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $components
   *   An array of field configuration components to render.
   * @param array $parent_keys
   *   (optional) An array of parent keys to preserve hierarchy in nested structures.
   */
  private function cfieldSettingsElements(array &$element, $form_state, array $components, $parent_keys = []) {
    $all_components = $this->castorcitoManager->castorcitoComponents($components);
    $components_cfields_settings = $this->getSetting('components_cfields_settings');

    foreach ($all_components as $c_key => $component) {
      $current_keys = [...$parent_keys, $c_key];
      $cfields_current_keys = [...$current_keys, 'cfields'];
      $cfield_configurations = $component->get('field_configuration');
      $component_element = [
        '#type' => 'details',
        '#title' => $component->label(),
      ];

      if (empty($parent_keys)) {
        $component_element['#states'] = [
          'visible' => [
            ':input[value="' . $c_key . '"]' => ['checked' => TRUE],
          ],
        ];
      }

      $this->castorcitoManager->setNestedArrayValue($element['components_cfields_settings'], $current_keys, $component_element);
      $this->castorcitoManager->setNestedArrayValue($element['components_cfields_settings'], $cfields_current_keys, [
        '#type' => 'fieldset',
        '#title' => $this->t('CFields'),
      ]);

      foreach ($cfield_configurations as $f_key => $configuration) {
        $cfield_current_keys = [...$cfields_current_keys, $f_key];
        $cfield_current_setting_keys = [...$cfield_current_keys, 'settings'];

        $this->castorcitoManager->setNestedArrayValue($element['components_cfields_settings'], $cfield_current_keys, [
          '#type' => 'details',
          '#title' => $this->t($configuration['label']),
        ]);

        $this->castorcitoManager->setNestedArrayValue($element['components_cfields_settings'], $cfield_current_setting_keys, [
          '#type' => 'container',
        ]);

        if (in_array($configuration['id'], self::CONTAINERS)) {
          if ($configuration['id'] === 'advanced_container') {
            $cfield_head_container_current_keys = [...$cfield_current_setting_keys, 'allowed_head_component'];
            $this->castorcitoManager->setNestedArrayValue($element['components_cfields_settings'], $cfield_head_container_current_keys, [
              '#type' => 'details',
              '#title' => $this->t('Head component'),
            ]);

            $head_component = [$configuration['settings']['head_component']];
            $this->cfieldSettingsElements($element, $form_state, $head_component, $cfield_head_container_current_keys);
          }

          $cfield_container_current_keys = [...$cfield_current_setting_keys, 'allowed_components'];
          $this->castorcitoManager->setNestedArrayValue($element['components_cfields_settings'], $cfield_container_current_keys, [
            '#type' => 'details',
            '#title' => $this->t('Allowed components'),
          ]);

          $children_components = $configuration['settings']['allowed_children'];
          $this->cfieldSettingsElements($element, $form_state, $children_components, $cfield_container_current_keys);
        }

        // CFields settings.
        $settings_default_values = $this->castorcitoManager->getNestedArrayValue($components_cfields_settings, $cfield_current_setting_keys, $configuration['settings']);
        $settings_elements = $this->buildSettingsElements($form_state, $configuration, $settings_default_values);
        $this->castorcitoManager->setNestedArrayValue($element['components_cfields_settings'], $cfield_current_setting_keys, $settings_elements);

        // CFields general.
        $default_values = $this->castorcitoManager->getNestedArrayValue($components_cfields_settings, $cfield_current_keys, $configuration);
        $this->castorcitoManager->setNestedArrayValue(
          $element['components_cfields_settings'],
          $cfield_current_keys,
          $this->buildGeneralElements($default_values)
        );
      }
    }
  }

  /**
   * Builds general form elements for field widget configuration.
   *
   * @param array $default_values
   * @return array
   */
  private function buildGeneralElements(array $default_values) {
    return [
      'description' => [
        '#type' => 'textarea',
        '#title' => $this->t('Help text'),
        '#default_value' => $default_values['description'],
        '#rows' => 5,
        '#description' => $this->t('Instructions to present to the user below this field on the editing form.<br />Allowed HTML tags: @tags', ['@tags' => FieldFilteredMarkup::displayAllowedTags()]) . '<br />',
      ],
    ];
  }

  /**
   * Builds custom settings form elements for the field widget.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param array $configuration
   * @param array $default_values
   * @return array
   */
  private function buildSettingsElements(FormStateInterface $form_state, array $configuration, array $default_values) {
    $form = [];
    $cfield = $this->componentFieldManager->createInstance($configuration['id']);
    $overridable_configuration = $cfield->overridableConfiguration();

    if (empty($overridable_configuration)) {
      return [];
    }

    $subform_state = SubformState::createForSubform($form, $form, $form_state);
    $build_configuration_form = $cfield->buildConfigurationForm($form, $subform_state);
    $cfield_configurations = array_intersect_key(
      $build_configuration_form,
      array_flip($overridable_configuration)
    );

    if (empty($cfield_configurations)) {
      return [];
    }

    return $this->setDefaultValues($cfield_configurations, $default_values);
  }

  /**
   * Sets default values for a group of form fields.
   *
   * @param array $fields
   * @param array $default_values
   * @return array $fields
   */
  private function setDefaultValues(array $fields, array $default_values) {
    foreach ($default_values as $key => $value) {
      if (isset($fields[$key])) {
        if (is_array($value) && is_array($fields[$key]) && $fields[$key]['#type'] != 'checkboxes') {
          $fields[$key] = $this->setDefaultValues($fields[$key], $value);
        }
        elseif (isset($fields[$key]['#default_value'])) {
          $fields[$key]['#default_value'] = $value;
        }
      }
    }

    return $fields;
  }

}
